#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `data2passcal` package."""

from __future__ import division, print_function

import ftplib
import os
import sys
import unittest

from data2passcal.data2passcal import get_FTP, ismseed, scan_dir, send2passcal
from data2passcal.data2passcal import FTP_CONNECT_ATTEMPTS, FTP_SEND_ATTEMPTS

if sys.version_info < (3, 3):
    from mock import patch
else:
    from unittest.mock import patch

VERSION = '2020.119'

MOCK_TEST = True


class TestData2passcal(unittest.TestCase):
    """Tests for `data2passcal` package."""

    def setUp(self):
        """Set up test fixtures, if any"""
        dir_testdata = os.path.dirname(os.path.realpath(__file__)) + '/test_data' # noqa
        filelist = [x for x in scan_dir(dir_testdata)
                    if not os.path.basename(x).startswith('.') and not
                    os.path.basename(x).endswith('.log')]
        self.dir_testdata = dir_testdata
        self.filelist = filelist

    def test_scan_dir(self):
        """
        Test basic functionality of scan_dir function
        Only 5 files available under ./test_data directory
        Not taking into account .DS_STORE file
        """
        filelist = [x for x in scan_dir(self.dir_testdata)
                    if not os.path.basename(x).startswith('.') and not
                    os.path.basename(x).endswith('.log')]
        self.assertEqual(len(filelist), 5, 'Incorrect number of files')

    def test_ismseed(self):
        """Test basic functionality of ismseed function"""
        for f in self.filelist:
            self.assertTrue(ismseed(f), '{} is not a miniseed file'
                            .format(os.path.basename(f)))

    @patch('data2passcal.data2passcal.ftplib.FTP', autospec=True)
    def test_get_FTP_mock(self, mock_ftp_constructor):
        """Mock test creating ftp connection to PASSCAL"""
        mock_ftp = mock_ftp_constructor.return_value
        get_FTP()
        self.assertLess(mock_ftp_constructor.call_count, FTP_CONNECT_ATTEMPTS,
                        'Number of ftp connection attempts exceeeds {}'
                        .format(FTP_CONNECT_ATTEMPTS))
        mock_ftp.quit()

    @patch('data2passcal.data2passcal.ftplib.FTP', autospec=True)
    def test_send_data_mock(self, mock_ftp_constructor):
        """Mock test sending MSEED files (test data) to PASSCAL's QC system"""
        mock_ftp = mock_ftp_constructor.return_value
        send2passcal(self.filelist)
        self.assertTrue(mock_ftp.storbinary.called, 'No data sent')
        self.assertEqual(mock_ftp.storbinary.call_count, len(self.filelist),
                         'Failed to send all files - Sent {0} of {1}'
                         .format(mock_ftp.storbinary.call_count,
                                 len(self.filelist)))
        files_sent = []
        for x in mock_ftp.storbinary.call_args_list:
            args, kwargs = x
            files_sent.append(args[0].split(' ')[1])
        for f in self.filelist:
            f = os.path.basename(f)
            self.assertLess(files_sent.count(f), FTP_SEND_ATTEMPTS,
                            'Attempted to send file {0} more than {1} times'
                            .format(f, FTP_SEND_ATTEMPTS))

    @unittest.skipIf(MOCK_TEST == True, "skipping real send2passcal test")
    def test_send_data(self):
        """Test sending MSEED files (test data) to PASSCAL's QC system"""
        ftp = get_FTP()
        send2passcal(self.filelist)
        wdir = ftp.pwd()
        try:
            files_sent = [os.path.basename(x) for x in ftp.nlst(wdir)]
        except ftplib.error_perm() as resp:
            if str(resp) == "550 No files found":
                print("No files found in this directory")
            else:
                raise
        for f in self.filelist:
            f_ = os.path.basename(f)
            self.assertIn(f_, files_sent, 'File {} was not sent to PASSCAL'
                          .format(f_))
        ftp.quit()


if __name__ == '__main__':
    unittest.main()
